unit UDIBSurf;

interface

//uses
 // USprite;

type
 TGDIBitMapInfoHeader = record
   biSize: Longint;
   biWidth: Longint;
   biHeight: Longint;
   biPlanes: Word;
   biBitCount: Word;
   biCompression: Longint;
   biSizeImage: Longint;
   biXPelsPerMeter: Longint;
   biYPelsPerMeter: Longint;
   biClrUsed: Longint;
   biClrImportant: Longint;
 end;

 TGDIPaletteEntry = record
   peRed: Byte;
   peGreen: Byte;
   peBlue: Byte;
   peFlags: Byte;
 end;

 TPalEntries   = array[0..255] of TGDIPaletteEntry;
 TMyLogPalette = record
    palVersion    : Word;
    palNumEntries : Word;
    palEntry      : TPalEntries;
  end;

  TPalIndices    = array[0..255] of word;
  PMyBitmapInfo  = ^TMyBitmapInfo;
  TMyBitmapInfo  = record
    BMIheader   : TGDIBitMapInfoHeader;
    BMIcolors   : TPalIndices;
  end;

var
  DIBbits: Pointer;
  DIBhandle: Integer;
  DIBheader: TMyBitmapInfo;
  OLDPalette,
  DIBhPalette: Integer;
  DIBDC: Integer;
  DIBWidth: integer;
  DIBHeight: integer;
  DIBWidth_b: integer;
  DIBSize: integer;
  OLDHandle: Integer;
  aColorLookUpTable: array[0..1023] of Byte;
procedure CreateDIBSurf(oNewHPalette:INteger; aW, aH: Word);

procedure DestroyDIBSurf;
procedure FillSurface(nColor: Byte);
function gdiCreatePalette(const LogPalette: TMyLogPalette): Integer; stdcall;
function gdiSelectPalette(DC: Integer; Palette: Integer; ForceBackground: Integer): Integer; stdcall;
function gdiCreateCompatibleDC(DC: Integer): Integer; stdcall;
function gdiSelectObject(DC: Integer; p2: Integer): Integer; stdcall;
function gdiDeleteObject(p1: Integer): Integer; stdcall;
function CreateDIBSection(DC: Integer; const p2: TMyBitmapInfo; p3: Integer;
  var p4: Pointer; p5: Pointer; p6: LongInt): Integer; stdcall;
function gdiDeleteDC(DC: Integer): Integer; stdcall;
function gdiBitBlt(DestDC: Integer; X, Y, nWidth, Height: Integer; SrcDC: Integer;
  XSrc, YSrc: Integer; Rop: LongInt): Integer; stdcall;

procedure PutSprite(X, Y : Integer; nDelta: Byte; aData: Pointer); assembler;
procedure xPutSprite(X, Y: Integer; nColor, nFlag: Byte; K: Pointer); assembler;
procedure xPutColorSprite(X, Y: Integer; nColor: Word; nFlag: Byte; K: Pointer); assembler;
procedure xPutColorNoTransSprite(X, Y: Integer; nColor: Word; nFlag: Byte; K: Pointer); assembler;
procedure PutTransSprite(X, Y : Integer; nDelta: Byte; aData: Pointer); assembler;
procedure PutNoTransSprite(X, Y : Integer; nDelta: Byte; aData: Pointer); assembler;

implementation


type
  AByte = array[0..0] of Byte;
  PByte = ^AByte;
const
  BI_RGB = 0;
  BI_RLE8 = 1;
  BI_RLE4 = 2;
  BI_BITFIELDS = 3;
  DIB_RGB_COLORS = 0;     { color table in RGBs  }
  DIB_PAL_COLORS = 1;     { color table in palette indices  }
  SRCCOPY     = $00CC0020;     { dest = source                    }
  gdi32 = 'GDI32.DLL';


function gdiCreatePalette; external gdi32 name 'CreatePalette';
function gdiSelectPalette; external gdi32 name 'SelectPalette';
function gdiCreateCompatibleDC; external gdi32 name 'CreateCompatibleDC';
function gdiSelectObject; external gdi32 name 'SelectObject';
function gdiDeleteObject; external gdi32 name 'DeleteObject';
function CreateDIBSection; external gdi32 name 'CreateDIBSection';
function gdiDeleteDC; external gdi32 name 'DeleteDC';
function gdiBitBlt; external gdi32 name 'BitBlt';




procedure CreateDIBSurf(oNewHPalette: Integer; aW, aH: Word);
var lp1 : integer;
begin
  DIBbits      := nil;
  DIBhandle    := 0;
  DIBhpalette  := oNewHPalette;
  DIBDC        := gdiCreateCompatibleDC(0);
  OLDPalette   := gdiSelectPalette(DIBDC, DIBhpalette, 0);
  with DIBheader do begin
    with bmiHeader do begin
      biSize          := sizeof(TGDIBitMapInfoHeader);
      biWidth         := aW;
      biHeight        := aH;
      biPlanes        := 1;
      biBitCount      := 8;
      biCompression   := BI_RGB;
      biSizeImage     := 0;
      biXPelsPerMeter := 0;
      biYPelsPerMeter := 0;
      biClrUsed       := 0;
      biClrImportant  := 0;
    end;
    for lp1:=0 to 255 do BMIcolors[lp1] := (lp1+0) and 255; { pal_indices }
  end;

  DIBWidth   := aW;
  DIBHeight  := aH;
  DIBWidth_b := ((DIBWidth+3) shr 2) shl 2;
  DIBSize    := DIBWidth_b*DIBHeight;
  DIBWidth   := ((DIBWidth+3) shr 2) shl 2;

  if OLDHandle<>0 then gdiSelectObject(DIBDC, OLDHandle);

  if DIBhandle<>0 then gdiDeleteObject(DIBhandle);

  DIBheader.BMIheader.biWidth  := DIBWidth;
  DIBheader.BMIheader.biHeight :=-DIBHeight;  { Top down for me please...}
  { the minus sign may or may not be necessary on your machine - I'm not sure }
  { without it - the DIB is bottom up - See old WinG documentation            }
  DIBhandle := CreateDIBSection(DIBDC, DIBheader ,DIB_PAL_COLORS,DIBbits,nil,0);
  OLDHandle := gdiSelectObject(DIBDC,DIBhandle);
end;

procedure DestroyDIBSurf;
begin
  if OLDHandle<>0 then gdiSelectObject(DIBDC, OLDHandle);
  // if OLDPalette<>0 then gdiSelectPalette(DIBDC, OLDPalette , 0);
  if DIBhandle<>0    then gdiDeleteObject(DIBhandle);
  if DIBhpalette<>0  then gdiDeleteObject(DIBhpalette);
  gdiDeleteDC(DIBDC);
end;




procedure PutSprite(X, Y : Integer; nDelta: Byte; aData: Pointer);
var
  nColor: Byte;
asm
  mov nColor, nDelta
  push esi
  push edi
  push ebx

  mov  ebx, X
  mov  edi, DIBBits
  mov  eax, DIBWidth_b
  mul  Y
  add  ebx, eax

  mov  esi, aData
  mov  eax, 8


  xor  edx, edx

@LoopLin:
  push edx

  xor  ecx, ecx

@LoopCol:
  mov  dl, byte ptr esi[eax]
  or   dl, dl
  jz   @Color0
  add  dl, nColor
@Color0:  
  mov  byte ptr edi[ebx], dl

@DontPutPix:
  inc  ecx
  inc  ebx
  inc  eax

  cmp  ecx, dword ptr esi[0]
  jl   @LoopCol

  sub  ebx, dword ptr esi[0]
  add  ebx, DIBWidth_b

  pop  edx
  inc  edx
  cmp  edx, dword ptr esi[4]
  jl   @LoopLin

  pop  ebx
  pop  edi
  pop  esi
end;


procedure xPutSprite(X, Y: Integer; nColor, nFlag: Byte; K: Pointer); assembler;
var
  nC, nF: Byte;
asm
  push esi
  push edi
  push ebx

  mov  nC, nColor
  mov  cl, nFlag
  mov  nF, cl
  mov  esi, K

  mov  ecx, X
  mov  edx, Y
  mov  edi, 8

  xor  ebx, ebx
@LoopLin:

  xor  eax, eax
@LoopCol:

  push eax
  push ebx
  push ecx
  push edx

  test nF, 10000000b
  jz   @NotInvertX
  add  ecx, dword ptr esi[0] { X = X + W  }
  sub  ecx, eax              { X = X + W - i }
  mov  eax, ecx
  jmp  @InvertXDone
@NotInvertX:
  add  eax, ecx              { X = X + i }
@InvertXDone:

  test nF, 01000000b
  jz   @NotInvertY
  add  edx, dword ptr esi[4] { Y = Y + H }
  sub  edx, ebx              { Y = Y + H - j }
  mov  ebx, edx
  jmp  @InvertYDone
@NotInvertY:
  add  ebx, edx   { Y = Y + j }
@InvertYDone:

  shl  ebx, 8     { Y = Y * 256 }
  add  eax, ebx   { X = Y * 256 + X}

  mov  bl, byte ptr esi[edi]
  or   bl, bl
  jz   @DontPutPixel
  add  bl, nC

  push esi
  mov  esi, DIBBits
  mov  byte ptr esi[eax], bl
  pop  esi

@DontPutPixel:

  pop  edx
  pop  ecx
  pop  ebx
  pop  eax

  inc  edi

  inc  eax
  cmp  eax, dword ptr esi[0]
  jl   @LoopCol

  inc  ebx
  cmp  ebx, dword ptr esi[4]
  jl   @LoopLin

  pop  ebx
  pop  edi
  pop  esi
end;


procedure xPutColorSprite(X, Y: Integer; nColor: Word; nFlag: Byte; K: Pointer); assembler;
var
  nC: word;
  nF: Byte;
asm
  push esi
  push edi
  push ebx

  mov  nC, nColor
  mov  cl, nFlag
  mov  nF, cl
  mov  esi, K

  mov  ecx, X
  mov  edx, Y
  mov  edi, 8

  xor  ebx, ebx
@LoopLin:

  xor  eax, eax
@LoopCol:

  push eax
  push ebx
  push ecx
  push edx

  test nF, 10000000b
  jz   @NotInvertX
  add  ecx, dword ptr esi[0] { X = X + W  }
  sub  ecx, eax              { X = X + W - i }
  dec  ecx
  mov  eax, ecx
  jmp  @InvertXDone
@NotInvertX:
  add  eax, ecx              { X = X + i }
@InvertXDone:

  test nF, 01000000b
  jz   @NotInvertY
  add  edx, dword ptr esi[4] { Y = Y + H }
  sub  edx, ebx              { Y = Y + H - j }
  dec  edx
  mov  ebx, edx
  jmp  @InvertYDone
@NotInvertY:
  add  ebx, edx   { Y = Y + j }
@InvertYDone:

  shl  ebx, 8     { Y = Y * 256 }
  add  eax, ebx   { X = Y * 256 + X}

  xor  ebx, ebx
  mov  bl, byte ptr esi[edi]
  or   bl, bl
  jz   @DontPutPixel
  xor  bh, bh
  add  bx, nC
  mov  bl, byte ptr aColorLookUpTable[ebx]
  add  bl, 10

  push esi
  mov  esi, DIBBits
  mov  byte ptr esi[eax], bl
  pop  esi

@DontPutPixel:

  pop  edx
  pop  ecx
  pop  ebx
  pop  eax

  inc  edi

  inc  eax
  cmp  eax, dword ptr esi[0]
  jl   @LoopCol

  inc  ebx
  cmp  ebx, dword ptr esi[4]
  jl   @LoopLin

  pop  ebx
  pop  edi
  pop  esi
end;

procedure xPutColorNoTransSprite(X, Y: Integer; nColor: Word; nFlag: Byte; K: Pointer); assembler;
var
  nC: word;
  nF: Byte;
asm
  push esi
  push edi
  push ebx

  mov  nC, nColor
  mov  cl, nFlag
  mov  nF, cl
  mov  esi, K

  mov  ecx, X
  mov  edx, Y
  mov  edi, 8

  xor  ebx, ebx
@LoopLin:

  xor  eax, eax
@LoopCol:

  push eax
  push ebx
  push ecx
  push edx

  test nF, 10000000b
  jz   @NotInvertX
  add  ecx, dword ptr esi[0] { X = X + W  }
  sub  ecx, eax              { X = X + W - i }
  dec  ecx
  mov  eax, ecx
  jmp  @InvertXDone
@NotInvertX:
  add  eax, ecx              { X = X + i }
@InvertXDone:

  test nF, 01000000b
  jz   @NotInvertY
  add  edx, dword ptr esi[4] { Y = Y + H }
  sub  edx, ebx              { Y = Y + H - j }
  dec  edx
  mov  ebx, edx
  jmp  @InvertYDone
@NotInvertY:
  add  ebx, edx   { Y = Y + j }
@InvertYDone:

  shl  ebx, 8     { Y = Y * 256 }
  add  eax, ebx   { X = Y * 256 + X}

  xor  ebx, ebx
  mov  bl, byte ptr esi[edi]
  add  bx, nC
  mov  bl, byte ptr aColorLookUpTable[ebx]
  add  bl, 10

  push esi
  mov  esi, DIBBits
  mov  byte ptr esi[eax], bl
  pop  esi

  pop  edx
  pop  ecx
  pop  ebx
  pop  eax

  inc  edi

  inc  eax
  cmp  eax, dword ptr esi[0]
  jl   @LoopCol

  inc  ebx
  cmp  ebx, dword ptr esi[4]
  jl   @LoopLin

  pop  ebx
  pop  edi
  pop  esi
end;



procedure PutTransSprite(X, Y : Integer; nDelta: Byte; aData: Pointer); assembler;
var
  nColor: Byte;
asm

  mov nColor, nDelta
  push esi
  push edi
  push ebx

  mov  ebx, X
  mov  edi, DIBBits
  mov  eax, DIBWidth_b
  mul  Y
  add  ebx, eax

  mov  esi, aData
  mov  eax, 8


  xor  edx, edx

@LoopLin:
  push edx

  xor  ecx, ecx

@LoopCol:
  mov  dl, byte ptr esi[eax]
  or   dl, dl
  jz   @DontPutPix

  add  dl, nColor
  mov  byte ptr edi[ebx], dl

@DontPutPix:
  inc  ecx
  inc  ebx
  inc  eax

  cmp  ecx, dword ptr esi[0]
  jl   @LoopCol

  sub  ebx, dword ptr esi[0]
  add  ebx, DIBWidth_b

  pop  edx
  inc  edx
  cmp  edx, dword ptr esi[4]
  jl   @LoopLin

  pop  ebx
  pop  edi
  pop  esi

end;

procedure PutNoTransSprite(X, Y : Integer; nDelta: Byte; aData: Pointer); assembler;
var
  nColor: Byte;
asm
  mov nColor, nDelta
  push esi
  push edi
  push ebx

  mov  ebx, X
  mov  edi, DIBBits
  mov  eax, DIBWidth_b
  mul  Y
  add  ebx, eax

  mov  esi, aData
  mov  eax, 8


  xor  edx, edx

@LoopLin:
  push edx

  xor  ecx, ecx

@LoopCol:
  mov  dl, byte ptr esi[eax]
  add  dl, nColor
  mov  byte ptr edi[ebx], dl

@DontPutPix:
  inc  ecx
  inc  ebx
  inc  eax

  cmp  ecx, dword ptr esi[0]
  jl   @LoopCol

  sub  ebx, dword ptr esi[0]
  add  ebx, DIBWidth_b

  pop  edx
  inc  edx
  cmp  edx, dword ptr esi[4]
  jl   @LoopLin

  pop  ebx
  pop  edi
  pop  esi
end;
procedure FillSurface(nColor: Byte);
begin
  FillChar(DIBbits^, DIBSize, nColor);
end;

end.

